<?php

/**
 * OK HTTP result code
 */
define('APACHE_RC_OK', 200);

/**
 * Partial content HTTP result code
 */
define('APACHE_RC_PARTIALCONTENT', 206);

/**
 * Not modified HTTP result code
 */
define('APACHE_RC_NOMOD', 304);

/**
 * Apache LogParser:: implementation.
 *
 * $Horde: odin/lib/LogParser/apache.php,v 1.3 2004/05/23 17:09:25 jan Exp $
 *
 * @author  Sagi Bashari <sagi@boom.org.il>
 * @version $Revision: 1.3 $
 * @access  public
 * @since   Odin 0.0.2
 * @package Odin
 */
class LogParser_apache extends LogParser {

    /**
     * Only log these HTTP result codes.
     *
     * @var array
     * @access public
     */
    var $log_rc = array(APACHE_RC_OK,
                        APACHE_RC_PARTIALCONTENT,
                        APACHE_RC_NOMOD);

    /**
     * Only log these HTTP commands.
     *
     * @var array
     * @access public
     */
    var $log_commands = array('GET', 'HEAD', 'POST', 'TRACE');

    /**
     * Directory indexes, log these as '/'.
     *
     * @var array
     * @access public
     */
    var $directory_index = array('index.php', 'index.php3', 'index.phtml', 'index.html', 'index.htm', 'index.cgi');

    /**
     * Do not log files with these extensions.
     *
     * @var array
     * @access public
     */
    var $nolog_extensions = array('css', 'jpg', 'jpeg', 'gif', 'png');
     
    /**
     * Holding script names as keys with array of HTTP GET variables
     * that should be stored - as their values.
     *
     * @var array
     * @see storeGETVariable()
     */
    var $GET_variables = array();

    /**
     * Holding user agents counter (key=agent, value=counter).
     * @access public
     * @var array
     */
    var $stats_useragent = array();

    /**
     * Holding referer counters (key=referer, value=counter).
     *
     * @var array
     * @access public
     */
    var $stats_referer = array();

    /**
     * Holding the hit counters.
     *
     * Contains array of years that contains array of months, days,
     * and hours.  For example - if you want to look up how many hits
     * were at February 15 2002 between 16:00 and 17:00, you would use
     * $stats_url[2002][2][15][16].
     *
     * @var array
     * @access public
     */
    var $stats_url = array();

    /**
     * Constructs a new LogParser_apache object.
     *
     * @access public
     */
    function LogParser_apache()
    {
    }

    /**
     * Parses log file.
     *
     * @access public
     * @param string $filename Filename to parse.
     * @returns mixed PEAR_Error class on failure, or true otherwise.
     */
    function parseFile($filename)
    {
        if (!($fp = fopen($filename, 'r'))) {
            return PEAR::raiseError('Unable to open file for reading: '.$fp);
        }

        while (($buf = fgets($fp, 4096)) !== false) {
            $buf = trim($buf);
            $this->parseRecord($buf);
        }

        fclose($fp);

        return false;
    }

    /**
     * Store an HTTP GET variable.
     *
     * Tells log parser not to strip this variable from this script.
     * Any other variables will be stripped.
     * Use '/' for directory indexes as their filename will be stripped.
     *
     * @access public
     * @param string $script Script name, use full path (ie. '/script.php').
     * @param string $variable Variable name (ie. 'id' for 'script.php?id=5').
     */
    function storeGETVariable($script, $variable)
    {
        $this->GET_variables[$script][] = $variable;
    }

    /**
     * Parses log record.
     *
     * @param string $record Combined log record.
     */
    function parseRecord($record)
    {
        // parse into variables
        sscanf($record, '%s %s %s [%[^]]] "%[^"]" %s %s "%[^"]" "%[^"]"',
            $host, $logname, $user, $strtime, $request, $status, $bytes, $referer, $useragent
        );

        // only log results codes that are set in $log_rc
        if (!in_array($status, $this->log_rc)) {
            return;
        }

        // convert apache time to unixtime
        $strtime = str_replace('/', ' ', $strtime);
        $strtime = explode(':', $strtime, 2);
        $strtime = $strtime[0].' '.$strtime[1];
        $time = strtotime($strtime);
        
        // parse request
        $arr = explode(' ', $request);
        $command = $arr[0];
        $script = $arr[1];

        if (!in_array($command, $this->log_commands)) {
            return;
        }
        
        // strip protocol
        $pos = strpos($script, '://');

        if ($pos !== false) {
            $script = substr($script, ($pos + 3));

            // strip domain
            $pos = strpos($script, '/');
            $script = substr($script, $pos);
        }
    
        // split GET variables from the script name
        $arr = explode('?', $script);
        $script_name = $arr[0];
        $script_vars = isset($arr[1]) ? $arr[1] : '';
        
        // if the script name is a directory index, replace it with '/'
        $script_basename = basename($script_name);

        if (in_array($script_basename, $this->directory_index)) {
            $script_name = substr($script_name, 0, (strlen($script_name) - strlen($script_basename)));
        }
        
        // check if we shouldn't log this extension
        $arr = explode('.', $script_basename);
        if (isset($arr[1])) {
            $extension = $arr[1];
            if (in_array($extension, $this->nolog_extensions)) {
                return;
            }
        }

        /* now rebuild the script path - use the new $script_name as
           filename, only include variables that were added through
           storeGETVariable(). */
        $script = $script_name;

        if (!empty($script_vars) && isset($this->GET_variables[$script_name])) {
            $first = true;
    
            foreach (explode('&', $script_vars) as $var) {
                $arr = explode('=',$var);
                $var_name = $arr[0];
                
                if (in_array($var_name, $this->GET_variables[$script_name])) {
                    if ($first) {
                        $script .= '?';
                        $first = false;
                    } else {
                        $script .= '&';
                    }
                
                    $script .= $var;
                }
            }
        }

        // strip variables from referer
        $arr = explode('?', $referer);
        $referer = $arr[0];
        
        // useragent stats
        if (!isset($this->stats_useragent[$useragent])) {
            $this->stats_useragent[$useragent] = 0;
        }

        $this->stats_useragent[$useragent]++;

        // url hits stats
        $date_year = (int)date('Y',$time);
        $date_month = (int)date('m',$time);
        $date_day = (int)date('d',$time);
        $date_hour = (int)date('G',$time);

        if (!isset($this->stats_url[$date_year])) {
            $this->stats_url[$date_year] = array();
        }

        if (!isset($this->stats_url[$date_year][$date_month])) {
            $this->stats_url[$date_year][$date_month] = array();
        }

        if (!isset($this->stats_url[$date_year][$date_month][$date_day])) {
            $this->stats_url[$date_year][$date_month][$date_day] = array();
        }

        if (!isset($this->stats_url[$date_year][$date_month][$date_day][$date_hour])) {
            $this->stats_url[$date_year][$date_month][$date_day][$date_hour] = array();
        }
        
        if (!isset($this->stats_url[$date_year][$date_month][$date_day][$date_hour][$script])) {
            $this->stats_url[$date_year][$date_month][$date_day][$date_hour][$script] = 0;
        }

        $this->stats_url[$date_year][$date_month][$date_day][$date_hour][$script]++;

        // referer stats
        if (!empty($referer) && $referer != '-') {
            if (!isset($this->stats_referer[$referer])) {
                $this->stats_referer[$referer] = 0;
            }

            $this->stats_referer[$referer]++;
        }
    }

}
